#include <arpa/inet.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include "netgame.h"

int initialize_server(const int port)
{
	struct sockaddr_in srv_addr;
	struct protoent *pe;
	srv_addr.sin_family = AF_INET;
	srv_addr.sin_addr.s_addr = INADDR_ANY;

	/* cislo portu */
	srv_addr.sin_port = htons(port);

	/* Prevod jmena na cislo protokolu */
	if(!(pe = getprotobyname("tcp"))) {
		fprintf(stderr, "%s: Unknown protocol\n", "tcp");
		exit(EXIT_FAILURE);
	}
	    
	/* Vytvoreni a inicializace serveroveho socketu */
	int server = socket(PF_INET, SOCK_STREAM, pe->p_proto);
	if(server < 0) {
		perror("socket()");
		exit(EXIT_FAILURE);
	}
	/* Potlaceni chyby "Socket already in use" pri restartu serveru */
	int opt = 1;
	if(setsockopt(server, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt))) {
		perror("setsockopt()");
		exit(EXIT_FAILURE);
	}
	if(bind(server, (struct sockaddr *)&srv_addr, sizeof(srv_addr))) {
		perror("bind()");
		exit(EXIT_FAILURE);
	}
	if(listen(server, 0)) {
		perror("listen()");
		exit(EXIT_FAILURE);
	}

	struct sockaddr_in cl_addr;
	/* Prijeti noveho klienta */
	socklen_t addr_len = sizeof(cl_addr);
	int client = accept(server, (struct sockaddr *)&cl_addr, &addr_len);
	if(client < 0) {
		perror("accept()");
		/* continue; */
	}

	/* Prevod IP adresy a DNS jmena do retezce */
	struct hostent *he;
	char *ip, *name;
	ip = inet_ntoa(cl_addr.sin_addr);
	if(!(he = gethostbyaddr((const char *)&cl_addr.sin_addr, 
				sizeof(cl_addr.sin_addr), AF_INET)))
	{
		herror("client address");
		name = ip;
	} else
		name = he->h_name;
	/* Informativni hlaska o pripojeni klienta */
	printf("client=%s/%s:%hu fd=%d\n", name, ip, ntohs(cl_addr.sin_port), client);
	return client;
}

int initialize_client(const char *host, const int port)
{
	/* viz httpcl2_tcp.c */
	struct hostent *he;
	struct sockaddr_in srv_addr;
	struct protoent *pe;
	srv_addr.sin_family = AF_INET;

	/* Prevod jmena na IP adresu */
	if(!(he = gethostbyname(host))) {
		herror(host);
		exit(EXIT_FAILURE);
	}
	memcpy(&srv_addr.sin_addr.s_addr, he->h_addr_list[0],
			sizeof(srv_addr.sin_addr.s_addr));

	/* cislo portu */
	srv_addr.sin_port = htons(port);

	/* Prevod jmena na cislo protokolu */
	if(!(pe = getprotobyname("tcp"))) {
		fprintf(stderr, "%s: Unknown protocol\n", "tcp");
		exit(EXIT_FAILURE);
	}
    
	/* Vytvoreni socketu a navazani spojeni se serverem */
	int server = socket(PF_INET, SOCK_STREAM, pe->p_proto);
	if(server < 0) {
		perror("socket()");
		exit(EXIT_FAILURE);
	}
	if(connect(server, (struct sockaddr *)&srv_addr, sizeof(srv_addr))) {
		fprintf(stderr, "%s:%d: %s\n", host, port, strerror(errno));
		exit(EXIT_FAILURE);
	}
	return server;
}

/* Zapis zpravy b do socketu client s kontrolou chyb */
int netwrite(int sock, char *b)
{
	char *p = b;
	ssize_t w;
	int len = strlen(b);

	while(len > 0) {
		if((w = write(sock, p, len)) < 0) {
			printf("sock=%d Sending response: %s\n", sock, strerror(errno));
			return -1;
		}
		p += w;
		len -= w;
	}

	return 0;
}

int netread(int sock, char *b)
{
	int len;
	char buff[BUFFSIZE];

	strcpy(b, "");
	while((len= read(sock, buff, BUFFSIZE)) > 0){
		if(buff[len-1]=='$'){
			buff[len-1]='\0';
			strcat(b, buff);
		//	printf("DEBUG:netread: precetl jsem: %s\n", b);
			break;
		}else{
			buff[len]='\0';
			strcat(b, buff);
		//	printf("DEBUG:netread: precetl jsem: %s\n", b);
		}
	}
	if(len == 0) {
		close(sock);
		printf("Hra ukoncena druhou stranou.\n");
		exit(EXIT_SUCCESS);
	}
	if(len < 0){
		perror("read(sock)");
		exit(EXIT_FAILURE);
	}
	return EXIT_SUCCESS;
}

void terminate_server(const int client, const int server)
{
	close(client);
	close(server);
}

void terminate_client(const int client)
{
	close(client);
}


